##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Functional Tests for Onlinehelp

"""
import os
import transaction
import unittest

from zope.site.interfaces import IRootFolder
from zope.app.file import File
from zope.app.onlinehelp.tests.test_onlinehelp import testdir
from zope.app.onlinehelp import globalhelp
from zope.app.onlinehelp.testing import OnlineHelpLayer

from webtest import TestApp

class BrowserTestCase(unittest.TestCase):

    layer = OnlineHelpLayer

    def setUp(self):
        super(BrowserTestCase, self).setUp()
        self._testapp = TestApp(self.layer.make_wsgi_app())

    def checkForBrokenLinks(self, orig_response, path, basic=None):
        response = self.publish(path, basic=basic)
        links = response.html.find_all('a')

        for link in links:
            href = link.attrs['href']
            if '++apidoc++' in href:
                # We don't install this at test time
                continue
            if href.startswith('http://dev.zope.org'):
                # Don't try to follow external links
                continue

            self.publish(href, basic=basic)

    def publish(self, path, basic=None, form=None, headers=None):
        assert basic
        self._testapp.authorization = ('Basic', tuple(basic.split(':')))
        env = {'wsgi.handleErrors': False}
        if form:
            response = self._testapp.post(path, params=form,
                                          extra_environ=env, headers=headers)
        else:
            response = self._testapp.get(path, extra_environ=env, headers=headers)

        response.getBody = lambda: response.unicode_normal_body
        response.getStatus = lambda: response.status_int
        response.getHeader = lambda n: response.headers[n]
        return response



class TestBrowser(BrowserTestCase):

    def test_contexthelp(self):
        path = os.path.join(testdir(), 'help.txt')
        globalhelp.registerHelpTopic('help', 'Help', '', path, IRootFolder)
        path = os.path.join(testdir(), 'help2.txt')
        globalhelp.registerHelpTopic('help2', 'Help2', '', path, IRootFolder,
            'contents.html')

        transaction.commit()

        response = self.publish("/+/action.html", basic='mgr:mgrpw',
                                form={
                                    'type_name': u'zope.app.content.File',
                                    'id': u'file'
                                })

        self.assertEqual(response.getStatus(), 302)

        response = self.publish('/contents.html', basic='mgr:mgrpw')

        self.assertEqual(response.getStatus(), 200)
        body = ' '.join(response.getBody().split())
        self.assertIn(
            "javascript:popup('contents.html/++help++/@@contexthelp.html",
            body)

        response = self.publish(
            '/contents.html/++help++/@@contexthelp.html', basic='mgr:mgrpw')

        self.assertEqual(response.getStatus(), 200)
        body = ' '.join(response.getBody().split())
        self.assertIn("This is another help!", body)

        response = self.publish('/index.html/++help++/@@contexthelp.html',
                                basic='mgr:mgrpw')

        self.assertEqual(response.getStatus(), 200)
        body = ' '.join(response.getBody().split())
        self.assertIn("This is a help!", body)

        response = self.publish('/file/edit.html/++help++/@@contexthelp.html',
                                basic='mgr:mgrpw')

        self.assertEqual(response.getStatus(), 200)
        body = ' '.join(response.getBody().split())
        self.assertIn(
            "Welcome to the Zope 3 Online Help System.",
            body)

        path = '/contents.html/++help++'
        response = self.publish(path, basic='mgr:mgrpw')

        self.assertEqual(response.getStatus(), 200)
        body = ' '.join(response.getBody().split())
        self.assertIn("Topics", body)

        self.checkForBrokenLinks(body, path, basic='mgr:mgrpw')

class TestZPT(unittest.TestCase):

    layer = OnlineHelpLayer

    def test_render(self):
        from zope.app.onlinehelp.browser import ZPTOnlineHelpTopicView
        from zope.publisher.browser import TestRequest
        from zope.location.interfaces import LocationError
        from zope.app.rotterdam import Rotterdam
        from zope.publisher.skinnable import applySkin

        class Context(object):
            path = os.path.join(os.path.dirname(__file__),
                                'helptopic.pt')
            title = 'title'

        request = TestRequest()
        applySkin(request, Rotterdam)

        # Normally these are used for IZPTOnlineHelpTopic objects,
        # which have a `path` attribute to a custom template file.
        # We're going to use it with `helptopic.pt`, which is a template
        # used for OnlineHelpTopic objects and won't work for us:
        zpt = ZPTOnlineHelpTopicView(Context, request)
        with self.assertRaises(LocationError) as e:
            zpt.renderTopic()
        self.assertEqual(e.exception.args[1], 'topicContent')

        # We have to assign a 'topicContent' to be able to use that template:
        zpt.topicContent = "the topic text"
        zpt.renderTopic()

class TestContextHelpView(unittest.TestCase):

    def test_idempotent(self):
        from zope.app.onlinehelp.browser import ContextHelpView
        view = ContextHelpView(None, None)
        view.topic = self

        self.assertIs(self, view.getContextHelpTopic())

    def test_without_view(self):
        from zope.app.onlinehelp.browser import ContextHelpView
        class Context(object):
            @property
            def context(self):
                return self
        context = Context()
        view = ContextHelpView(context, None)
        topic = view.getContextHelpTopic()
        self.assertIs(context, topic)

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""`OnlineHelp` views
"""
__docformat__ = 'restructuredtext'

from zope.component import createObject, getMultiAdapter

from zope.publisher.browser import BrowserView
from zope.publisher.interfaces.browser import IBrowserView
from zope.traversing.api import getName, getParent

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from zope.app.onlinehelp import getTopicFor


class OnlineHelpTopicView(BrowserView):
    """View for one particular help topic."""

    def topicContent(self):
        """ render the source of the help topic """
        source = createObject(self.context.type, self.context.source)
        view = getMultiAdapter((source, self.request))
        html = view.render()
        return html

    renderTopic = ViewPageTemplateFile('helptopic.pt')


class ZPTOnlineHelpTopicView(BrowserView):
    """View for a page template based help topic."""

    def renderTopic(self):
        """Render the registred topic."""
        path = self.context.path
        view = ViewPageTemplateFile(path)
        return view(self)


class ContextHelpView(BrowserView):

    def __init__(self, context, request):
        super(ContextHelpView, self).__init__(context, request)
        self.topic = None

    def getContextualTopicView(self):
        """Retrieve and render the source of a context help topic """
        topic = self.getContextHelpTopic()
        view = getMultiAdapter((topic, self.request), name='index.html')
        return view.renderTopic()

    def getContextHelpTopic(self):
        """Retrieve a help topic based on the context of the
        help namespace.

        If the context is a view, try to find
        a matching help topic for the view and its context.
        If no help topic is found, try to get a help topic for
        the context only.

        If the context is not a view, try to retrieve a help topic
        based on the context.

        If nothing is found, return the onlinehelp root topic
        """
        if self.topic is not None:
            return self.topic

        onlinehelp = self.context
        help_context = onlinehelp.context
        self.topic = None
        if IBrowserView.providedBy(help_context):
            # called from a view
            self.topic = getTopicFor(
                getParent(help_context),
                getName(help_context)
                )
            if self.topic is None:
                # nothing found for view try context only
                self.topic = getTopicFor(getParent(help_context))
        else:
            # called without view
            self.topic = getTopicFor(help_context)

        if self.topic is None:
            self.topic = onlinehelp

        return self.topic

    contextHelpTopic = property(getContextHelpTopic)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Onlinehelp tree view Tests

"""
import os

import unittest

from zope import component
from zope.pagetemplate.tests.util import check_xml
from zope.publisher.browser import TestRequest
from zope.app.component.testing import PlacefulSetup
from zope.app.onlinehelp.tests import util
from zope.app.onlinehelp.interfaces import IOnlineHelp
from zope.app.onlinehelp.onlinehelp import OnlineHelp
from zope.app.onlinehelp.onlinehelptopic import OnlineHelpTopic
from zope.app.onlinehelp.browser.tree import OnlineHelpTopicTreeView


def testdir():
    import zope.app.onlinehelp.tests
    return os.path.dirname(zope.app.onlinehelp.tests.__file__)


class TestOnlineHelpTopicTreeView(PlacefulSetup, unittest.TestCase):

    def setUp(self):
        PlacefulSetup.setUp(self, site=True)
        path = os.path.join(testdir(), 'help.txt')
        self.onlinehelp = OnlineHelp('Help', path)
        component.provideUtility(self.onlinehelp, IOnlineHelp, "OnlineHelp")

    def test_onlinehelp(self):
        view = OnlineHelpTopicTreeView
        treeView = view(self.rootFolder, TestRequest()).getTopicTree
        check_xml(treeView(), util.read_output('test1.xml'))

    def test_topics(self):
        path = os.path.join(testdir(), 'help.txt')

        id = 'topic1'
        title = 'Topic1'
        parentPath = ""
        topic1 = OnlineHelpTopic(id, title, path, parentPath)
        self.onlinehelp['topic1'] = topic1

        id = 'topic1_1'
        title = 'Topic1_1'
        parentPath = 'topic1'
        topic1_1 = OnlineHelpTopic(id, title, path, parentPath)
        topic1['topic1_1']  = topic1_1

        id = 'topic1_1_1'
        title = 'Topic1_1_1'
        parentPath = 'topic1/topic1_1'
        topic1_1_1 = OnlineHelpTopic(id, title, path, parentPath)
        topic1_1['topic1_1_1']  = topic1_1_1

        id = 'topic2'
        title = 'Topic2'
        parentPath = ""
        topic2 = OnlineHelpTopic(id, title, path, parentPath)
        self.onlinehelp['topic2'] = topic2

        view = OnlineHelpTopicTreeView
        treeView = view(self.rootFolder, TestRequest()).getTopicTree
        check_xml(treeView(), util.read_output('test2.xml'))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
# 
##############################################################################
"""Meta-Configuration Handlers for "help" namespace.

These handlers process the `registerTopic()` directive of
the "help" ZCML namespace.

$Id$
"""
__docformat__ = 'restructuredtext'

from zope.app.onlinehelp import globalhelp


class OnlineHelpTopicDirective(object):

    def __init__(self, _context, id, title, parent="", doc_path=None, 
        for_=None, view=None, class_=None, resources=None):
        self._context = _context
        self.id = id
        self.title = title
        self.parent = parent
        self.doc_path = doc_path
        self.for_ = for_
        self.view = view
        self.class_ = class_
        self.resources = resources
        
    def _args(self):
        return (self.parent, self.id, self.title, self.doc_path, self.for_,
                self.view, self.class_, self.resources)

    def _discriminator(self):
        return ('registerHelpTopic', self.parent, self.id)

    def __call__(self):
        self._context.action(
            discriminator=self._discriminator(),
            callable=globalhelp.registerHelpTopic,
            args=self._args(),
            order=666666,
        )



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the gts ZCML namespace directives.

"""
import unittest

from zope.interface import Interface
from zope.configuration import xmlconfig
from zope.configuration.xmlconfig import XMLConfig
from zope.component.interfaces import IFactory
from zope.component.factory import Factory
from zope.security.interfaces import IPermission
from zope.security.permission import Permission

import zope.traversing
import zope.app.component
import zope.app.security
import zope.app.onlinehelp
from zope.app.onlinehelp import tests
ztapi = tests
from zope.app.onlinehelp import globalhelp
from zope.app.onlinehelp.onlinehelptopic import OnlineHelpTopic
from zope.app.onlinehelp.onlinehelptopic import RESTOnlineHelpTopic
from zope.app.onlinehelp.onlinehelptopic import STXOnlineHelpTopic
from zope.app.onlinehelp.onlinehelptopic import ZPTOnlineHelpTopic

from zope.component import testing


class I1(Interface):
    pass


class DirectivesTest(testing.PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super(DirectivesTest, self).setUp()
        ztapi.provideUtility(IPermission, Permission('zope.View', 'View', ''),
                             'zope.View')
        XMLConfig('meta.zcml', zope.app.security)()
        XMLConfig('meta.zcml', zope.app.component)()
        XMLConfig('meta.zcml', zope.app.onlinehelp)()
        XMLConfig('configure.zcml', zope.traversing)

        default = Factory(OnlineHelpTopic)
        rest = Factory(RESTOnlineHelpTopic)
        stx = Factory(STXOnlineHelpTopic)
        zpt = Factory(ZPTOnlineHelpTopic)
        ztapi.provideUtility(IFactory, default, 'onlinehelp.topic.default')
        ztapi.provideUtility(IFactory, rest, 'onlinehelp.topic.rest')
        ztapi.provideUtility(IFactory, stx, 'onlinehelp.topic.stx')
        ztapi.provideUtility(IFactory, zpt, 'onlinehelp.topic.zpt')

    def test_register(self):
        self.assertEqual(list(globalhelp.keys()), [])
        XMLConfig('help.zcml', tests)()
        res = [u'help4', u'help5', u'help2', u'help3', u'help1']
        res.sort()

        helpList = sorted(globalhelp.keys())
        self.assertEqual(helpList, res)
        topic = globalhelp['help1']
        self.assertIn('test1.png', topic.keys())

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main()
